@@ -74,7 +74,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
     else:
         logging.getLogger("paramiko").setLevel(logging.WARN)
 
-    sshinfo = vmtools.get_clean_builder('builder', options.reset_server)
+    sshinfo = vmtools.get_clean_builder('builder')
 
     output = None
     try:
@@ -83,6 +83,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
                 buildserverid = subprocess.check_output(['vagrant', 'ssh', '-c',
                                                          'cat /home/vagrant/buildserverid'],
                                                         cwd='builder').strip().decode()
+                status_output['buildserverid'] = buildserverid
                 logging.debug(_('Fetched buildserverid from VM: {buildserverid}')
                               .format(buildserverid=buildserverid))
             except Exception as e:
@@ -140,8 +141,8 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
         ftp.chdir(homedir)
 
         ftp.put(os.path.join(serverpath, '..', 'buildserver',
-                             'config.buildserver.py'), 'config.py')
-        ftp.chmod('config.py', 0o600)
+                             'config.buildserver.yml'), 'config.yml')
+        ftp.chmod('config.yml', 0o600)
 
         # Copy over the ID (head commit hash) of the fdroidserver in use...
         with open(os.path.join(os.getcwd(), 'tmp', 'fdroidserverid'), 'wb') as fp:
@@ -202,8 +203,12 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
             send_dir(lib)
             # Copy the metadata file too...
             ftp.chdir(posixpath.join(homedir, 'srclibs'))
-            ftp.put(os.path.join('srclibs', name + '.txt'),
-                    name + '.txt')
+            srclibsfile = os.path.join('srclibs', name + '.yml')
+            if os.path.isfile(srclibsfile):
+                ftp.put(srclibsfile, os.path.basename(srclibsfile))
+            else:
+                raise BuildException(_('cannot find required srclibs: "{path}"')
+                                     .format(path=srclibsfile))
         # Copy the main app source code
         # (no need if it's a srclib)
         if (not basesrclib) and os.path.exists(build_dir):
@@ -267,7 +272,7 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
             ftp.get(toolsversion_log, os.path.join(log_dir, toolsversion_log))
             logging.debug('retrieved %s', toolsversion_log)
         except Exception as e:
-            logging.warn('could not get %s from builder vm: %s' % (toolsversion_log, e))
+            logging.warning('could not get %s from builder vm: %s' % (toolsversion_log, e))
 
         # Retrieve the built files...
         logging.info("Retrieving build output...")
@@ -290,7 +295,8 @@ def build_server(app, build, vcs, build_dir, output_dir, log_dir, force):
     finally:
         # Suspend the build server.
         vm = vmtools.get_build_vm('builder')
-        vm.suspend()
+        logging.info('destroying buildserver after build')
+        vm.destroy()
 
         # deploy logfile to repository web server
         if output:
@@ -323,6 +329,10 @@ def transform_first_char(string, method):
     return method(string[0]) + string[1:]
 
 
+def add_failed_builds_entry(failed_builds, appid, build, entry):
+    failed_builds.append([appid, int(build.versionCode), str(entry)])
+
+
 def get_metadata_from_apk(app, build, apkfile):
     """get the required metadata from the built APK
 
@@ -371,7 +381,8 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
         if build.sudo:
             logging.info("Running 'sudo' commands in %s" % os.getcwd())
 
-            p = FDroidPopen(['sudo', 'bash', '-x', '-c', build.sudo])
+            p = FDroidPopen(['sudo', 'DEBIAN_FRONTEND=noninteractive',
+                             'bash', '-x', '-c', build.sudo])
             if p.returncode != 0:
                 raise BuildException("Error running sudo command for %s:%s" %
                                      (app.id, build.versionName), p.output)
@@ -392,7 +403,7 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
             f.write(common.get_android_tools_version_log(build.ndk_path()))
     else:
         if build.sudo:
-            logging.warning('%s:%s runs this on the buildserver with sudo:\n\t%s'
+            logging.warning('%s:%s runs this on the buildserver with sudo:\n\t%s\nThese commands were skipped because fdroid build is not running on a dedicated build server.'
                             % (app.id, build.versionName, build.sudo))
 
     # Prepare the source code...
@@ -432,11 +443,6 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
 
         gradletasks += ['assemble' + flavours_cmd + 'Release']
 
-        if config['force_build_tools']:
-            force_gradle_build_tools(build_dir, config['build_tools'])
-            for name, number, libpath in srclibpaths:
-                force_gradle_build_tools(libpath, config['build_tools'])
-
         cmd = [config['gradle']]
         if build.gradleprops:
             cmd += ['-P' + kv for kv in build.gradleprops]
@@ -459,15 +465,14 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
 
         def del_dirs(dl):
             for d in dl:
-                if d in dirs:
-                    shutil.rmtree(os.path.join(root, d))
+                shutil.rmtree(os.path.join(root, d), ignore_errors=True)
 
         def del_files(fl):
             for f in fl:
                 if f in files:
                     os.remove(os.path.join(root, f))
 
-        if any(f in files for f in ['build.gradle', 'settings.gradle']):
+        if any(f in files for f in ['build.gradle', 'build.gradle.kts', 'settings.gradle', 'settings.gradle.kts']):
             # Even when running clean, gradle stores task/artifact caches in
             # .gradle/ as binary files. To avoid overcomplicating the scanner,
             # manually delete them, just like `gradle clean` should have removed
@@ -478,6 +483,7 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
                       os.path.join('build', 'outputs'),
                       os.path.join('build', 'reports'),
                       os.path.join('build', 'tmp'),
+                      os.path.join('buildSrc', 'build'),
                       '.gradle'])
             del_files(['gradlew', 'gradlew.bat'])
 
@@ -496,6 +502,7 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
     else:
         # Scan before building...
         logging.info("Scanning source for common problems...")
+        scanner.options = options  # pass verbose through
         count = scanner.scan_source(build_dir, build)
         if count > 0:
             if force:
@@ -524,8 +531,7 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
 
         # Substitute source library paths into commands...
         for name, number, libpath in srclibpaths:
-            libpath = os.path.relpath(libpath, root_dir)
-            cmd = cmd.replace('$$' + name + '$$', libpath)
+            cmd = cmd.replace('$$' + name + '$$', os.path.join(os.getcwd(), libpath))
 
         p = FDroidPopen(['bash', '-x', '-c', cmd], cwd=root_dir)
 
@@ -653,7 +659,7 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
 
         # expected to fail.
         # Signing will fail if not set by environnment vars (cf. p4a docs).
-        # But the unsigned apk will be ok.
+        # But the unsigned APK will be ok.
         p.returncode = 0
 
     elif bmethod == 'gradle':
@@ -678,9 +684,17 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
 
         bindir = os.path.join(root_dir, 'bin')
 
+    if os.path.isdir(os.path.join(build_dir, '.git')):
+        import git
+        commit_id = common.get_head_commit_id(git.repo.Repo(build_dir))
+    else:
+        commit_id = build.commit
+
     if p is not None and p.returncode != 0:
-        raise BuildException("Build failed for %s:%s" % (app.id, build.versionName), p.output)
-    logging.info("Successfully built version " + build.versionName + ' of ' + app.id)
+        raise BuildException("Build failed for %s:%s@%s" % (app.id, build.versionName, commit_id),
+                             p.output)
+    logging.info("Successfully built version {versionName} of {appid} from {commit_id}"
+                 .format(versionName=build.versionName, appid=app.id, commit_id=commit_id))
 
     omethod = build.output_method()
     if omethod == 'maven':
@@ -734,7 +748,7 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
             # really old path
             os.path.join(root_dir, 'build', 'apk'),
             ]
-        # If we build with gradle flavours with gradle plugin >= 3.0 the apk will be in
+        # If we build with gradle flavours with gradle plugin >= 3.0 the APK will be in
         # a subdirectory corresponding to the flavour command used, but with different
         # capitalization.
         if flavours_cmd:
@@ -776,10 +790,10 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
         raise BuildException("APK is debuggable")
 
     # By way of a sanity check, make sure the version and version
-    # code in our new apk match what we expect...
+    # code in our new APK match what we expect...
     logging.debug("Checking " + src)
     if not os.path.exists(src):
-        raise BuildException("Unsigned apk is not at expected location of " + src)
+        raise BuildException("Unsigned APK is not at expected location of " + src)
 
     if common.get_file_extension(src) == 'apk':
         vercode, version = get_metadata_from_apk(app, build, src)
@@ -789,8 +803,11 @@ def build_local(app, build, vcs, build_dir, output_dir, log_dir, srclib_dir, ext
                                   " Expected: '%s' / '%s'")
                                  % (version, str(vercode), build.versionName,
                                     str(build.versionCode)))
+        if (options.scan_binary or config.get('scan_binary')) and not options.skipscan:
+            if scanner.scan_binary(src):
+                raise BuildException("Found blacklisted packages in final apk!")
 
-    # Copy the unsigned apk to our destination directory for further
+    # Copy the unsigned APK to our destination directory for further
     # processing (by publish.py)...
     dest = os.path.join(output_dir, common.get_release_filename(app, build))
     shutil.copyfile(src, dest)
@@ -865,7 +882,7 @@ def parse_commandline():
 
     parser = argparse.ArgumentParser(usage="%(prog)s [options] [APPID[:VERCODE] [APPID[:VERCODE] ...]]")
     common.setup_global_opts(parser)
-    parser.add_argument("appid", nargs='*', help=_("applicationId with optional versionCode in the form APPID[:VERCODE]"))
+    parser.add_argument("appid", nargs='*', help=_("application ID with optional versionCode in the form APPID[:VERCODE]"))
     parser.add_argument("-l", "--latest", action="store_true", default=False,
                         help=_("Build only the latest version of each package"))
     parser.add_argument("-s", "--stop", action="store_true", default=False,
@@ -882,8 +899,8 @@ def parse_commandline():
                         help=argparse.SUPPRESS)
     parser.add_argument("--skip-scan", dest="skipscan", action="store_true", default=False,
                         help=_("Skip scanning the source code for binaries and other problems"))
-    parser.add_argument("--dscanner", action="store_true", default=False,
-                        help=_("Setup an emulator, install the APK on it and perform a Drozer scan"))
+    parser.add_argument("--scan-binary", action="store_true", default=False,
+                        help=_("Scan the resulting APK(s) for known non-free classes."))
     parser.add_argument("--no-tarball", dest="notarball", action="store_true", default=False,
                         help=_("Don't create a source tarball, useful when testing a build"))
     parser.add_argument("--no-refresh", dest="refresh", action="store_false", default=True,
@@ -913,6 +930,7 @@ config = None
 buildserverid = None
 fdroidserverid = None
 start_timestamp = time.gmtime()
+status_output = None
 timeout_event = threading.Event()
 
 
@@ -979,6 +997,11 @@ def main():
     else:
         also_check_dir = None
 
+    if options.onserver:
+        status_output = dict()  # HACK dummy placeholder
+    else:
+        status_output = common.setup_status_output(start_timestamp)
+
     repo_dir = 'repo'
 
     build_dir = 'build'
@@ -990,11 +1013,11 @@ def main():
 
     # Read all app and srclib metadata
     pkgs = common.read_pkg_args(options.appid, True)
-    allapps = metadata.read_metadata(not options.onserver, pkgs, options.refresh, sort_by_time=True)
+    allapps = metadata.read_metadata(pkgs, options.refresh, sort_by_time=True)
     apps = common.read_app_args(options.appid, allapps, True)
 
     for appid, app in list(apps.items()):
-        if (app.Disabled and not options.force) or not app.RepoType or not app.builds:
+        if (app.get('Disabled') and not options.force) or not app.get('RepoType') or not app.get('Builds', []):
             del apps[appid]
 
     if not apps:
@@ -1015,10 +1038,10 @@ def main():
 
     if options.latest:
         for app in apps.values():
-            for build in reversed(app.builds):
+            for build in reversed(app.get('Builds', [])):
                 if build.disable and not options.force:
                     continue
-                app.builds = [build]
+                app['Builds'] = [build]
                 break
 
     if options.wiki:
@@ -1028,8 +1051,10 @@ def main():
         site.login(config['wiki_user'], config['wiki_password'])
 
     # Build applications...
-    failed_apps = {}
+    failed_builds = []
     build_succeeded = []
+    status_output['failedBuilds'] = failed_builds
+    status_output['successfulBuilds'] = build_succeeded
     # Only build for 36 hours, then stop gracefully.
     endtime = time.time() + 36 * 60 * 60
     max_build_time_reached = False
@@ -1037,7 +1062,7 @@ def main():
 
         first = True
 
-        for build in app.builds:
+        for build in app.get('Builds', []):
             if time.time() > endtime:
                 max_build_time_reached = True
                 break
@@ -1060,6 +1085,7 @@ def main():
             tools_version_log = ''
             if not options.onserver:
                 tools_version_log = common.get_android_tools_version_log(build.ndk_path())
+                common.write_running_status_json(status_output)
             try:
 
                 # For the first build of a particular app, we need to set up
@@ -1085,7 +1111,7 @@ def main():
 
                     if app.Binaries is not None:
                         # This is an app where we build from source, and
-                        # verify the apk contents against a developer's
+                        # verify the APK contents against a developer's
                         # binary. We get that binary now, and save it
                         # alongside our built one in the 'unsigend'
                         # directory.
@@ -1151,7 +1177,7 @@ def main():
                 if options.stop:
                     logging.debug("Error encoutered, stopping by user request.")
                     common.force_exit(1)
-                failed_apps[appid] = vcse
+                add_failed_builds_entry(failed_builds, appid, build, vcse)
                 wikilog = str(vcse)
             except FDroidException as e:
                 with open(os.path.join(log_dir, appid + '.log'), 'a+') as f:
@@ -1166,7 +1192,7 @@ def main():
                 if options.stop:
                     logging.debug("Error encoutered, stopping by user request.")
                     common.force_exit(1)
-                failed_apps[appid] = e
+                add_failed_builds_entry(failed_builds, appid, build, e)
                 wikilog = e.get_wikitext()
             except Exception as e:
                 logging.error("Could not build app %s due to unknown error: %s" % (
@@ -1174,7 +1200,7 @@ def main():
                 if options.stop:
                     logging.debug("Error encoutered, stopping by user request.")
                     common.force_exit(1)
-                failed_apps[appid] = e
+                add_failed_builds_entry(failed_builds, appid, build, e)
                 wikilog = str(e)
 
             if options.wiki and wikilog:
@@ -1206,6 +1232,7 @@ def main():
                 timer.cancel()  # kill the watchdog timer
 
         if max_build_time_reached:
+            status_output['maxBuildTimeReached'] = True
             logging.info("Stopping after global build timeout...")
             break
 
@@ -1213,53 +1240,16 @@ def main():
         logging.info("success: %s" % (app.id))
 
     if not options.verbose:
-        for fa in failed_apps:
-            logging.info("Build for app %s failed:\n%s" % (fa, failed_apps[fa]))
-
-    # perform a drozer scan of all successful builds
-    if options.dscanner and build_succeeded:
-        from .dscanner import DockerDriver
-
-        docker = DockerDriver()
-
-        try:
-            for app in build_succeeded:
-
-                logging.info("Need to sign the app before we can install it.")
-                subprocess.call("fdroid publish {0}".format(app.id))
-
-                apk_path = None
-
-                for f in os.listdir(repo_dir):
-                    if f.endswith('.apk') and f.startswith(app.id):
-                        apk_path = os.path.join(repo_dir, f)
-                        break
-
-                if not apk_path:
-                    raise Exception("No signed APK found at path: {path}".format(path=apk_path))
-
-                if not os.path.isdir(repo_dir):
-                    logging.critical("directory does not exists '{path}'".format(path=repo_dir))
-                    common.force_exit(1)
-
-                logging.info("Performing Drozer scan on {0}.".format(app))
-                docker.perform_drozer_scan(apk_path, app.id, repo_dir)
-        except Exception as e:
-            logging.error(str(e))
-            logging.error("An exception happened. Making sure to clean up")
-        else:
-            logging.info("Scan succeeded.")
-
-        logging.info("Cleaning up after ourselves.")
-        docker.clean()
+        for fb in failed_builds:
+            logging.info('Build for app {}:{} failed:\n{}'.format(*fb))
 
     logging.info(_("Finished"))
     if len(build_succeeded) > 0:
         logging.info(ngettext("{} build succeeded",
                               "{} builds succeeded", len(build_succeeded)).format(len(build_succeeded)))
-    if len(failed_apps) > 0:
+    if len(failed_builds) > 0:
         logging.info(ngettext("{} build failed",
-                              "{} builds failed", len(failed_apps)).format(len(failed_apps)))
+                              "{} builds failed", len(failed_builds)).format(len(failed_builds)))
 
     if options.wiki:
         wiki_page_path = 'build_' + time.strftime('%s', start_timestamp)
@@ -1283,13 +1273,27 @@ def main():
                     if m:
                         txt += "* host RAM: %s\n" % m.group(1)
                         break
+        fdroid_path = os.path.realpath(os.path.join(os.path.dirname(__file__), '..'))
+        buildserver_config = os.path.join(fdroid_path, 'makebuildserver.config.py')
+        if os.path.isfile(buildserver_config) and os.access(buildserver_config, os.R_OK):
+            with open(buildserver_config) as configfile:
+                for line in configfile:
+                    m = re.search(r'cpus\s*=\s*([0-9].*)', line)
+                    if m:
+                        txt += "* guest processors: %s\n" % m.group(1)
+                    m = re.search(r'memory\s*=\s*([0-9].*)', line)
+                    if m:
+                        txt += "* guest RAM: %s MB\n" % m.group(1)
         txt += "* successful builds: %d\n" % len(build_succeeded)
-        txt += "* failed builds: %d\n" % len(failed_apps)
+        txt += "* failed builds: %d\n" % len(failed_builds)
         txt += "\n\n"
         newpage.save(txt, summary='Run log')
         newpage = site.Pages['build']
         newpage.save('#REDIRECT [[' + wiki_page_path + ']]', summary='Update redirect')
 
+    if not options.onserver:
+        common.write_status_json(status_output)
+
     # hack to ensure this exits, even is some threads are still running
     common.force_exit()
 
