@@ -43,6 +43,9 @@ from .exception import VCSException, NoSubmodulesException, FDroidException, Met
 # required.
 def check_http(app):
 
+    ignoreversions = app.UpdateCheckIgnore
+    ignoresearch = re.compile(ignoreversions).search if ignoreversions else None
+
     try:
 
         if not app.UpdateCheckData:
@@ -61,7 +64,7 @@ def check_http(app):
         if len(urlcode) > 0:
             logging.debug("...requesting {0}".format(urlcode))
             req = urllib.request.Request(urlcode, None)
-            resp = urllib.request.urlopen(req, None, 20)
+            resp = urllib.request.urlopen(req, None, 20)  # nosec B310 scheme is filtered above
             page = resp.read().decode('utf-8')
 
             m = re.search(codeex, page)
@@ -74,7 +77,7 @@ def check_http(app):
             if urlver != '.':
                 logging.debug("...requesting {0}".format(urlver))
                 req = urllib.request.Request(urlver, None)
-                resp = urllib.request.urlopen(req, None, 20)
+                resp = urllib.request.urlopen(req, None, 20)  # nosec B310 scheme is filtered above
                 page = resp.read().decode('utf-8')
 
             m = re.search(verex, page)
@@ -82,8 +85,13 @@ def check_http(app):
                 raise FDroidException("No RE match for version")
             version = m.group(1)
 
-        return (version, vercode)
-
+        if ignoresearch and version:
+            if not ignoresearch(version):
+                return (version, vercode)
+            else:
+                return (None, ("Version {version} is ignored").format(version=version))
+        else:
+            return (version, vercode)
     except FDroidException:
         msg = "Could not complete http check for app {0} due to unknown error: {1}".format(app.id, traceback.format_exc())
         return (None, msg)
@@ -160,10 +168,11 @@ def check_tags(app, pattern):
                 if vercode:
                     logging.debug("Manifest exists in subdir '{0}'. Found version {1} ({2})"
                                   .format(subdir, version, vercode))
-                    if int(vercode) > int(hcode):
+                    i_vercode = common.version_code_string_to_int(vercode)
+                    if i_vercode > common.version_code_string_to_int(hcode):
                         hpak = package
                         htag = tag
-                        hcode = str(int(vercode))
+                        hcode = str(i_vercode)
                         hver = version
 
         if not hpak:
@@ -212,8 +221,8 @@ def check_repomanifest(app, branch=None):
             vcs.gotorevision(None)
 
         last_build = metadata.Build()
-        if len(app.builds) > 0:
-            last_build = app.builds[-1]
+        if len(app.get('Builds', [])) > 0:
+            last_build = app.get('Builds', [])[-1]
 
         try_init_submodules(app, last_build, vcs)
 
@@ -286,7 +295,7 @@ def check_gplay(app):
     headers = {'User-Agent': 'Mozilla/5.0 (X11; Linux i686; rv:18.0) Gecko/20100101 Firefox/18.0'}
     req = urllib.request.Request(url, None, headers)
     try:
-        resp = urllib.request.urlopen(req, None, 20)
+        resp = urllib.request.urlopen(req, None, 20)  # nosec B310 URL base is hardcoded above
         page = resp.read().decode()
     except urllib.error.HTTPError as e:
         return (None, str(e.code))
@@ -317,7 +326,7 @@ def try_init_submodules(app, last_build, vcs):
         try:
             vcs.initsubmodules()
         except NoSubmodulesException:
-            logging.info("No submodules present for {}".format(app.Name))
+            logging.info("No submodules present for {}".format(_getappname(app)))
 
 
 # Return all directories under startdir that contain any of the manifest
@@ -325,7 +334,7 @@ def try_init_submodules(app, last_build, vcs):
 def dirs_with_manifest(startdir):
     for root, dirs, files in os.walk(startdir):
         if any(m in files for m in [
-                'AndroidManifest.xml', 'pom.xml', 'build.gradle']):
+                'AndroidManifest.xml', 'pom.xml', 'build.gradle', 'build.gradle.kts']):
             yield root
 
 
@@ -349,9 +358,18 @@ def possible_subdirs(app):
             yield subdir
 
 
+def _getappname(app):
+    return common.get_app_display_name(app)
+
+
+def _getcvname(app):
+    return '%s (%s)' % (app.CurrentVersion, app.CurrentVersionCode)
+
+
 def fetch_autoname(app, tag):
 
-    if not app.RepoType or app.UpdateCheckMode in ('None', 'Static'):
+    if not app.RepoType or app.UpdateCheckMode in ('None', 'Static') \
+       or app.UpdateCheckName == "Ignore":
         return None
 
     if app.RepoType == 'srclib':
@@ -383,7 +401,7 @@ def fetch_autoname(app, tag):
         if new_name != app.AutoName:
             app.AutoName = new_name
             if not commitmsg:
-                commitmsg = "Set autoname of {0}".format(common.getappname(app))
+                commitmsg = "Set autoname of {0}".format(_getappname(app))
     else:
         logging.debug("...couldn't get autoname")
 
@@ -449,7 +467,7 @@ def checkupdates_app(app):
         if noverok:
             logging.info(logmsg)
         else:
-            logging.warn(logmsg)
+            logging.warning(logmsg)
     elif vercode == app.CurrentVersionCode:
         logging.info("...up to date")
     else:
@@ -462,29 +480,29 @@ def checkupdates_app(app):
     commitmsg = fetch_autoname(app, tag)
 
     if updating:
-        name = common.getappname(app)
-        ver = common.getcvname(app)
+        name = _getappname(app)
+        ver = _getcvname(app)
         logging.info('...updating to version %s' % ver)
-        commitmsg = 'Update CV of %s to %s' % (name, ver)
+        commitmsg = 'Update CurrentVersion of %s to %s' % (name, ver)
 
     if options.auto:
         mode = app.AutoUpdateMode
         if not app.CurrentVersionCode:
-            logging.warn("Can't auto-update app with no current version code: " + app.id)
+            logging.warning("Can't auto-update app with no CurrentVersionCode: " + app.id)
         elif mode in ('None', 'Static'):
             pass
         elif mode.startswith('Version '):
             pattern = mode[8:]
+            suffix = ''
             if pattern.startswith('+'):
                 try:
-                    suffix, pattern = pattern.split(' ', 1)
+                    suffix, pattern = pattern[1:].split(' ', 1)
                 except ValueError:
-                    raise MetaDataException("Invalid AUM: " + mode)
-            else:
-                suffix = ''
+                    raise MetaDataException("Invalid AutoUpdateMode: " + mode)
+
             gotcur = False
             latest = None
-            for build in app.builds:
+            for build in app.get('Builds', []):
                 if int(build.versionCode) >= int(app.CurrentVersionCode):
                     gotcur = True
                 if not latest or int(build.versionCode) > int(latest.versionCode):
@@ -497,17 +515,17 @@ def checkupdates_app(app):
                 newbuild = copy.deepcopy(latest)
                 newbuild.disable = False
                 newbuild.versionCode = app.CurrentVersionCode
-                newbuild.versionName = app.CurrentVersion + suffix
+                newbuild.versionName = app.CurrentVersion + suffix.replace('%c', newbuild.versionCode)
                 logging.info("...auto-generating build for " + newbuild.versionName)
-                commit = pattern.replace('%v', newbuild.versionName)
+                commit = pattern.replace('%v', app.CurrentVersion)
                 commit = commit.replace('%c', newbuild.versionCode)
                 newbuild.commit = commit
-                app.builds.append(newbuild)
-                name = common.getappname(app)
-                ver = common.getcvname(app)
+                app['Builds'].append(newbuild)
+                name = _getappname(app)
+                ver = _getcvname(app)
                 commitmsg = "Update %s to %s" % (name, ver)
         else:
-            logging.warn('Invalid auto update mode "' + mode + '" on ' + app.id)
+            logging.warning('Invalid auto update mode "' + mode + '" on ' + app.id)
 
     if commitmsg:
         metadata.write_metadata(app.metadatapath, app)
@@ -521,6 +539,18 @@ def checkupdates_app(app):
                 raise FDroidException("Git commit failed")
 
 
+def status_update_json(processed, failed):
+    """Output a JSON file with metadata about this run"""
+
+    logging.debug(_('Outputting JSON'))
+    output = common.setup_status_output(start_timestamp)
+    if processed:
+        output['processed'] = processed
+    if failed:
+        output['failed'] = failed
+    common.write_status_json(output)
+
+
 def update_wiki(gplaylog, locallog):
     if config.get('wiki_server') and config.get('wiki_path'):
         try:
@@ -563,9 +593,9 @@ def main():
     global config, options
 
     # Parse command line...
-    parser = ArgumentParser(usage="%(prog)s [options] [APPID [APPID ...]]")
+    parser = ArgumentParser()
     common.setup_global_opts(parser)
-    parser.add_argument("appid", nargs='*', help=_("applicationId to check for updates"))
+    parser.add_argument("appid", nargs='*', help=_("application ID of file to operate on"))
     parser.add_argument("--auto", action="store_true", default=False,
                         help=_("Process auto-updates"))
     parser.add_argument("--autoonly", action="store_true", default=False,
@@ -600,28 +630,30 @@ def main():
             version, reason = check_gplay(app)
             if version is None:
                 if reason == '404':
-                    logging.info("{0} is not in the Play Store".format(common.getappname(app)))
+                    logging.info("{0} is not in the Play Store".format(_getappname(app)))
                 else:
-                    logging.info("{0} encountered a problem: {1}".format(common.getappname(app), reason))
+                    logging.info("{0} encountered a problem: {1}".format(_getappname(app), reason))
             if version is not None:
                 stored = app.CurrentVersion
                 if not stored:
                     logging.info("{0} has no Current Version but has version {1} on the Play Store"
-                                 .format(common.getappname(app), version))
+                                 .format(_getappname(app), version))
                 elif LooseVersion(stored) < LooseVersion(version):
                     logging.info("{0} has version {1} on the Play Store, which is bigger than {2}"
-                                 .format(common.getappname(app), version, stored))
+                                 .format(_getappname(app), version, stored))
                 else:
                     if stored != version:
                         logging.info("{0} has version {1} on the Play Store, which differs from {2}"
-                                     .format(common.getappname(app), version, stored))
+                                     .format(_getappname(app), version, stored))
                     else:
                         logging.info("{0} has the same version {1} on the Play Store"
-                                     .format(common.getappname(app), version))
+                                     .format(_getappname(app), version))
         update_wiki(gplaylog, None)
         return
 
     locallog = ''
+    processed = []
+    failed = dict()
     for appid, app in apps.items():
 
         if options.autoonly and app.AutoUpdateMode in ('None', 'Static'):
@@ -634,13 +666,15 @@ def main():
 
         try:
             checkupdates_app(app)
+            processed.append(appid)
         except Exception as e:
             msg = _("...checkupdate failed for {appid} : {error}").format(appid=appid, error=e)
             logging.error(msg)
             locallog += msg + '\n'
+            failed[appid] = str(e)
 
     update_wiki(None, locallog)
-
+    status_update_json(processed, failed)
     logging.info(_("Finished"))
 
 
