@@ -25,14 +25,14 @@ options = None
 
 def main():
 
-    parser = ArgumentParser(usage="%(prog)s")
+    parser = ArgumentParser()
     common.setup_global_opts(parser)
     metadata.add_metadata_arguments(parser)
     options = parser.parse_args()
     metadata.warnings_action = options.W
     common.read_config(None)
 
-    metadata.read_metadata(xref=True)
+    metadata.read_metadata()
 
 
 if __name__ == "__main__":
